@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2005 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2011 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,10 +12,10 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
-"""Add the message to the list's current digest and possibly send it.
-"""
+"""Add the message to the list's current digest and possibly send it."""
 
 # Messages are accumulated to a Unix mailbox compatible file containing all
 # the messages destined for the digest.  This file must be parsable by the
@@ -29,6 +29,7 @@ import os
 import re
 import copy
 import time
+import traceback
 from types import ListType
 from cStringIO import StringIO
 
@@ -88,9 +89,21 @@ def process(mlist, msg, msgdata):
     if size / 1024.0 >= mlist.digest_size_threshhold:
         # This is a bit of a kludge to get the mbox file moved to the digest
         # queue directory.
-        mboxfp.seek(0)
-        send_digests(mlist, mboxfp)
-        os.unlink(mboxfile)
+        try:
+            # Enclose in try/except here because a error in send_digest() can
+            # silently stop regular delivery.  Unsuccessful digest delivery
+            # should be tried again by cron and the site administrator will be
+            # notified of any error explicitly by the cron error message.
+            mboxfp.seek(0)
+            send_digests(mlist, mboxfp)
+            os.unlink(mboxfile)
+        except Exception, errmsg:
+            # Bare except is generally prohibited in Mailman, but we can't
+            # forecast what exceptions can occur here.
+            syslog('error', 'send_digests() failed: %s', errmsg)
+            s = StringIO()
+            traceback.print_exc(file=s)
+            syslog('error', s.getvalue())
     mboxfp.close()
 
 
@@ -323,14 +336,16 @@ def send_i18n_digests(mlist, mboxfp):
                 uh = '\n\t'.join(uh.split('\n'))
                 print >> plainmsg, uh
         print >> plainmsg
-        payload = msg.get_payload(decode=True)\
+        # If decoded payload is empty, this may be multipart message.
+        # -- just stringfy it.
+        payload = msg.get_payload(decode=True) \
                   or msg.as_string().split('\n\n',1)[1]
         mcset = msg.get_content_charset('')
         if mcset and mcset <> lcset and mcset <> lcset_out:
             try:
                 payload = unicode(payload, mcset, 'replace'
                           ).encode(lcset, 'replace')
-            except LookupError:
+            except (UnicodeError, LookupError):
                 # TK: Message has something unknown charset.
                 #     _out means charset in 'outer world'.
                 payload = unicode(payload, lcset_out, 'replace'
@@ -346,16 +361,17 @@ def send_i18n_digests(mlist, mboxfp):
         footer['Content-Description'] = _('Digest Footer')
         mimemsg.attach(footer)
         # RFC 1153
-        # BAW: This is not strictly conformant RFC 1153.  The trailer is only
-        # supposed to contain two lines, i.e. the "End of ... Digest" line and
-        # the row of asterisks.  If this screws up MUAs, the solution is to
-        # add the footer as the last message in the RFC 1153 digest.  I just
-        # hate the way that VM does that and I think it's confusing to users,
-        # so don't do it unless there's a clamor.
+        # MAS: There is no real place for the digest_footer in an RFC 1153
+        # compliant digest, so add it as an additional message with
+        # Subject: Digest Footer
         print >> plainmsg, separator30
         print >> plainmsg
+        print >> plainmsg, 'Subject: ' + _('Digest Footer')
+        print >> plainmsg
         print >> plainmsg, footertxt
         print >> plainmsg
+        print >> plainmsg, separator30
+        print >> plainmsg
     # Do the last bit of stuff for each digest type
     signoff = _('End of ') + digestid
     # MIME
