@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2004 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2011 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """-request robot command queue runner."""
 
@@ -46,6 +46,10 @@ from email.MIMEText import MIMEText
 from email.MIMEMessage import MIMEMessage
 
 NL = '\n'
+CONTINUE = 0
+STOP = 1
+BADCMD = 2
+BADSUBJ = 3
 
 try:
     True, False
@@ -76,7 +80,8 @@ class Results:
         try:
             subj = make_header(decode_header(subj)).__unicode__()
             # TK: Currently we don't allow 8bit or multibyte in mail command.
-            subj = subj.encode('us-ascii')
+            # MAS: However, an l10n 'Re:' may contain non-ascii so ignore it.
+            subj = subj.encode('us-ascii', 'ignore')
             # Always process the Subject: header first
             self.commands.append(subj)
         except (HeaderParseError, UnicodeError, LookupError):
@@ -93,7 +98,7 @@ class Results:
         if part is None:
             # E.g the outer Content-Type: was text/html
             return
-        body = part.get_payload()
+        body = part.get_payload(decode=True)
         # text/plain parts better have string payloads
         assert isinstance(body, StringType) or isinstance(body, UnicodeType)
         lines = body.splitlines()
@@ -104,15 +109,19 @@ class Results:
     def process(self):
         # Now, process each line until we find an error.  The first
         # non-command line found stops processing.
-        stop = False
+        found = BADCMD
+        ret = CONTINUE
         for line in self.commands:
             if line and line.strip():
                 args = line.split()
                 cmd = args.pop(0).lower()
-                stop = self.do_command(cmd, args)
+                ret = self.do_command(cmd, args)
+                if ret == STOP or ret == CONTINUE:
+                    found = ret
             self.lineno += 1
-            if stop:
+            if ret == STOP or ret == BADCMD:
                 break
+        return found
 
     def do_command(self, cmd, args=None):
         if args is None:
@@ -135,8 +144,14 @@ class Results:
                 self.subjcmdretried += 1
                 cmd = args.pop(0)
                 return self.do_command(cmd, args)
-            return self.lineno <> 0
-        return handler.process(self, args)
+            if self.lineno <> 0:
+                return BADCMD
+            else:
+                return BADSUBJ
+        if handler.process(self, args):
+            return STOP
+        else:
+            return CONTINUE
 
     def send_response(self):
         # Helper
@@ -155,7 +170,7 @@ Attached is your original message.
         # Ignore empty lines
         unprocessed = [line for line in self.commands[self.lineno:]
                        if line and line.strip()]
-        if unprocessed:
+        if unprocessed and mm_cfg.RESPONSE_INCLUDE_LEVEL >= 2:
             resp.append(_('\n- Unprocessed:'))
             resp.extend(indent(unprocessed))
         if not unprocessed and not self.results:
@@ -164,7 +179,7 @@ Attached is your original message.
 No commands were found in this message.
 To obtain instructions, send a message containing just the word "help".
 """)))
-        if self.ignored:
+        if self.ignored and mm_cfg.RESPONSE_INCLUDE_LEVEL >= 2:
             resp.append(_('\n- Ignored:'))
             resp.extend(indent(self.ignored))
         resp.append(_('\n- Done.\n\n'))
@@ -190,12 +205,20 @@ To obtain instructions, send a message containing just the word "help".
             return
         msg = Message.UserNotification(
             recip,
-            self.mlist.GetBouncesEmail(),
+            self.mlist.GetOwnerEmail(),
             _('The results of your email commands'),
             lang=self.msgdata['lang'])
         msg.set_type('multipart/mixed')
         msg.attach(results)
-        orig = MIMEMessage(self.msg)
+        if mm_cfg.RESPONSE_INCLUDE_LEVEL == 1:
+            self.msg.set_payload(
+                _('Message body suppressed by Mailman site configuration\n'))
+        if mm_cfg.RESPONSE_INCLUDE_LEVEL == 0:
+            orig = MIMEText(_(
+                'Original message suppressed by Mailman site configuration\n'
+                ), _charset=charset)
+        else:
+            orig = MIMEMessage(self.msg)
         msg.attach(orig)
         msg.send(self.mlist)
 
@@ -235,17 +258,23 @@ class CommandRunner(Runner):
         # mylist-join, or mylist-leave, and the message metadata will contain
         # a key to which one was used.
         try:
+            ret = BADCMD
             if msgdata.get('torequest'):
-                res.process()
+                ret = res.process()
             elif msgdata.get('tojoin'):
-                res.do_command('join')
+                ret = res.do_command('join')
             elif msgdata.get('toleave'):
-                res.do_command('leave')
+                ret = res.do_command('leave')
             elif msgdata.get('toconfirm'):
                 mo = re.match(mm_cfg.VERP_CONFIRM_REGEXP, msg.get('to', ''))
                 if mo:
-                    res.do_command('confirm', (mo.group('cookie'),))
-            res.send_response()
-            mlist.Save()
+                    ret = res.do_command('confirm', (mo.group('cookie'),))
+            if ret == BADCMD and mm_cfg.DISCARD_MESSAGE_WITH_NO_COMMAND:
+                syslog('vette',
+                       'No command, message discarded, msgid: %s',
+                       msg.get('message-id', 'n/a'))
+            else:
+                res.send_response()
+                mlist.Save()
         finally:
             mlist.Unlock()
